from typing import (
    Optional,
    List,
)


class ServiceSeats(object):

    def __init__(
        self,
        seats,                 # type: str
        custom_seats=None,     # type: Optional[List[str]]
    ):
        # type: (...) -> None

        self.seats = seats
        self.custom_seats = custom_seats
