from typing import (
    Any,
    List,
)

from boss.model import User
from boss.model.repository import UserRepository
from .._BaseInteractor import BaseInteractor


class GetUsersListInteractor(BaseInteractor):

    def __init__(
        self,
        user_repository: UserRepository,
    ) -> None:

        self.user_repository = user_repository

    def execute(
        self,
        request: Any,
    ) -> List[User]:

        return self.user_repository.get_users_list()
