from typing import Optional


class CannotTotalOrderWithNotFilledMandatoryOption(Exception):

    def __init__(
        self,       # type: CannotTotalOrderWithNotFilledMandatoryOption
        part_code,  # type: int
        part_name,  # type: Optional[str]
    ):
        # type: (...) -> None
        super(CannotTotalOrderWithNotFilledMandatoryOption, self).__init__()

        self.part_code = part_code
        self.part_name = part_name
