INVALID_JSON = 0
UNEXPECTED_ERROR = -1
MISSING_REQUEST_NAME = -2
REPORT_FILE_NOT_FOUND = -3
REPORT_NOT_IMPLEMENTED = -4
ERROR_BUILDING_REPORT_DATA = -5
ERROR_BUILDING_DATA_BUILDER = -6
ERROR_FORMATTING_REPORT = -7
REPORT_FUNCTION_NOT_IMPLEMENTED = -8
INVALID_TEXT_FORMAT_PARAMETERS_COUNT = -9
TEXT_FORMAT_LENGTH_NOT_INTEGER = -10
INVALID_TEXT_FORMAT_SIDE_PARAMETER = -11
INVALID_CURRENCY_PARAMETERS_COUNT = -12
CURRENCY_VALUE_NOT_A_NUMBER = -13
INVALID_NUMBER_PARAMETERS_COUNT = -14
NUMBER_VALUE_NOT_A_NUMBER = -15
DATETIME_NOW_CANNOT_RECEIVE_PARAMETER = -16
INVALID_DATETIME_PARAMETERS_COUNT = -17
DATETIME_DATE_NOT_A_ISO_DATE = -18
I18N_LABEL_NOT_FOUND = -19
COULD_NOT_LOAD_POS_TRANSLATIONS = -20
CONDITION_IS_NOT_A_VARIABLE = -21
CONDITION_VARIABLE_NOT_PRESENT = -22
VALUE_RETURNED_FROM_OBJECT_LIST_IS_NOT_A_LIST = -23
VARIABLE_WAS_NOT_GENERATED_BY_THE_BUILDER = -24
TEXT_LIST_ATTRIBUTE_IS_NOT_A_LIST = -25
PADDING_IS_NOT_A_NUMBER = -26
INNER_LIST_ATTRIBUTE_IS_NOT_A_LIST = -27
INNER_LIST_MUST_HAVE_NAME = -28
ERROR_PARSING_REPORT_DATA = -29
