class CannotSaveEmptyOrder(Exception):

    def __init__(
        self,       # type: CannotSaveEmptyOrder
    ):
        # type: (...) -> None
        super(CannotSaveEmptyOrder, self).__init__()

    @staticmethod
    def get_error_code():
        # type: (...) -> str

        return "CannotSaveEmptyOrder"

    def __str__(
        self,  # type: CannotSaveEmptyOrder
    ):
        # type: (...) -> str

        return self.get_error_code()
