from typing import Optional


class LoyaltyDiscountData(object):
    def __init__(
        self,                       # type: LoyaltyDiscountData
        new_payment_amount=None,    # type: Optional[float]
        discount_amount=None,       # type: Optional[float]
        benefit_id=None,            # type: Optional[str]
        loyalty_id=None,            # type: Optional[str]
        benefit_key=None,           # type: Optional[str]
    ):
        # type: (...) -> None

        self.new_payment_amount = new_payment_amount
        self.discount_amount = discount_amount
        self.benefit_id = benefit_id
        self.loyalty_id = loyalty_id
        self.benefit_key = benefit_key
