from messagebus import (
    TokenPriority,
    TokenCreator,
)

event_manager_group = "5F"

TK_EVENT_MANAGER_PUBLISH = TokenCreator.create_token(
    priority=TokenPriority.high,
    message_group=event_manager_group,
    number="1",
)

TK_EVENT_MANAGER_GET_PENDING = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=event_manager_group,
    number="2",
)

TK_EVENT_MANAGER_GET_ABANDONED = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=event_manager_group,
    number="3",
)

TK_EVENT_MANAGER_GET_DETAILS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=event_manager_group,
    number="4",
)

TK_EVENT_MANAGER_DEFINITION_NOT_FOUND = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group=event_manager_group,
    number="F004",
)
