from application.domain import TableService  # noqa


class CloseTableInteractor(object):
    def __init__(self, table_service):
        # type: (TableService) -> None
        self.table_service = table_service

    def execute(
        self,           # type: CloseTableInteractor
        pos_id,         # type: str
        table_id,       # type: str
        user_id,        # type: str
    ):
        # type: (...) -> str

        return self.table_service.close_table(
            pos_id=pos_id,
            table_id=table_id,
            user_id=user_id
        )
