class Threshold(object):
    def __init__(self, value):
        # type: (unicode) -> None

        values = value.split("#")
        self.time = values[0]
        self.color = "#" + values[1]
        self.percentage = None
        if len(values) > 2:
            self.percentage = "#" + values[2]
