from dataclasses import dataclass


@dataclass
class AuthenticationData:

    generate_by: str
    user_id: str
    user_name: str
    long_name: str
    user_level: int
    authentication_key: str
    generation_date: str
    expiration_date: str
    interval: int
