class NoUpdateFound(Exception):
    pass


class ErrorApplyingUpdate(Exception):
    pass


class ConfigurationReaderException(Exception):
    pass


class ErrorInsertingNewUpdateVersion(Exception):
    pass


class ErrorRetrievingPendingUpdate(Exception):
    pass


class ErrorRetrievingUsers(Exception):
    pass


class ErrorCheckingGenesisDir(Exception):
    pass


class ErrorDownloadingUpdatePackage(Exception):
    pass


class ErrorNotifyingAppliedUpdate(Exception):
    pass


class HasNoProductsImagesToDownload(Exception):
    pass


class NotInsideUpdateWindow(Exception):
    pass


class ErrorCheckingPackageIntegrity(Exception):
    pass


class ErrorUpdatingStep(Exception):
    pass


class ErrorUpdatingStatus(Exception):
    pass


class ErrorRestoringBackup(Exception):
    pass


class ErrorUpdatingStartStop(Exception):
    pass


class ErrorCreatingBackup(Exception):
    pass


class ErrorRestartingHV(Exception):
    pass


class DisabledFunctionality(Exception):
    pass


class ErrorDeletingContent(Exception):
    pass


class ErrorExtractingLoaders(Exception):
    pass


class ErrorRecoveringFolder(Exception):
    pass


class ErrorUpdatingDatabases(Exception):
    pass


class ErrorGenesisDatabaseNotFound(Exception):
    pass


class ErrorDownloadingLoaders(Exception):
    pass


class ErrorDownloadingBinaries(Exception):
    pass


class RollbackError(Exception):
    pass


class InvalidDownloadUpdate(Exception):
    pass


class InvalidBOHConfigurations(Exception):
    pass


class SomeUserLogged(Exception):
    pass


class NotImplementedException(Exception):
    pass


class ErrorGettingTotemConfig(Exception):
    pass


class ErrorRetrievingJsonCatalog(Exception):
    pass


class DataBasesNotFound(Exception):
    pass


class ErrorValidatingOpenedUsers(Exception):
    pass


class ErrorDayIsOpen(Exception):
    pass


class ErrorGettingUsersList(Exception):
    pass


class ErrorUpdatingUserControlType(Exception):
    pass


class NoCatalogDirectoryFoundException(Exception):
    pass


class ErrorUpdatingUserRoles(Exception):
    pass


class NoCatalogJsonFoundException(Exception):
    pass


class NoCatalogUpdateFoundException(Exception):
    pass


class NotAllImagesDownload(Exception):
    pass


class ErrorUpdatingApache(Exception):
    pass


class ErrorIntegrityDatabase(Exception):
    pass


class CannotDeletePdvWithOrders(Exception):
    pass
