class GetTotaledMessageRequest(object):
    def __init__(
        self,
        pos_id,         # type: str
        customer_id,    # type: str
    ):
        # type: (...) -> None

        self.pos_id = int(pos_id)
        self.customer_id = customer_id
